#include "DistrhoPlugin.hpp"
#include <array>

struct Voice
{
    double hz;
    double phase;
};

constexpr unsigned int NUM_VOICES = 64;
constexpr unsigned int VOICE_MASK = NUM_VOICES - 1;

class Synth
{
public:
    explicit Synth(double sampleRate);
    void process(float *output, uint32_t frames);
    void setFrequencyShift(double in);
    void setTargetRatio(double in);
    void setSampleRate(double oldRate, double newRate);
    void setVolume(double in);

private:
    //Phase must persist between run invocation.
    //Phase of each voice, considered as a period function on the unit interval.
    std::array<double, NUM_VOICES> phases = {0};
    //Index of lowest voice in phase array.
    unsigned int idxFund = 0;
    float volume = 0.0f;
    double hzFund = 32.7;
    double hzShift = 1.0;
    double hzNyq = 24000.0;
    double ratio = 2.0;
    double targetRatio = 2.0;
    double ratioSlewRate = 0.99999;
    double sampleInterval = 1.0 / 48000.0;
};